#
# fishing.R, 14 Aug 22
#
# Data from:
# First Come First Served: The Impact of File Position on Code Review
# Enrico Fregnan, Larissa Braz, Marco D’Ambros, Alberto Bacchelli, Gül Çalikli

library("colorspace")
library("lme4")
library("plyr")

par(bty="l")
par(las=1)
par(pch="+")
pal_col=rainbow(2)


# Add cumulative lines of code
cum_lines_added=function(df)
{
t=df[order(df$file_position, decreasing=TRUE), ]
t$log_la=log(t$lines_added+0.1)
t$cum_log_la=cumsum(t$log_la)-t$log_la
t$prev_comments=cumsum(t$n_comments)-t$n_comments
t$prev_la=cumsum(t$lines_added)-t$lines_added
t$multi_commenters_in_pr=length(which(t$n_commenters > 1))
return(t)
}


met=read.csv("2208.04259/MetricsPRFileNoThread.csv.xz", as.is=TRUE)

# Add some totals for each pr
met=ddply(met, .(pr_id), cum_lines_added)

length(unique(met$pr_id))

# Some basic plots

pal_col=rainbow(5)
plot(count(met$files_in_pr), log="y", col=pal_col[1],
	xaxs="i",
	xlim=c(0, 100), ylim=c(1e3, 1e5),
	xlab="Files in pull request", ylab="Pull requests")
points(count(subset(met, n_comments_in_pr > 0)$files_in_pr), col=pal_col[2])
points(count(subset(met, n_comments_in_pr > 1)$files_in_pr), col=pal_col[3])
points(count(subset(met, n_comments_in_pr > 4)$files_in_pr), col=pal_col[4])
points(count(subset(met, n_comments_in_pr > 9)$files_in_pr), col=pal_col[5])

legend("topright", legend=c("All pull request", "At least 1 comment", "At least 2 comments",
				"At least 5 comments", "At least 10 comments"),
                        fill=pal_col, border="white", bty="n")

plot(count(met$file_position), log="xy",
	xlim=c(1, 100), ylim=c(1, 1e5))

table(met$is_test)
#   false    true 
# 2398186  491471 
table(met$n_comments)

plot(count(met$n_commenters), log="y",
	ylim=c(1, 2e6))

plot(count(met$n_comments), log="xy",
	ylim=c(1, 2e6))

# Subsets containing a small range of files
met20=subset(met, files_in_pr < 20)
met10=subset(met, files_in_pr < 10)

fe=count(met10$file_extension)
fe=fe[order(fe$freq, decreasing=TRUE), ]
top_fe=head(fe, n=20)$x

# Attempts to fit various models

met_mod=glm(n_comments ~ file_position+n_commenters, data=met10, family=poisson)
met_mod=glm(n_comments ~ file_position+n_commenters+I(n_commenters^2)+is_test,
				data=met10, family=poisson)
# Maximum of one commenter per file
met_mod=glm(n_comments ~ is_test+
				# cum_log_la+ # log prev_la is a better fit
				log(prev_la+0.1)+
				prev_comments+
				file_position+log(lines_added+0.1)+log(lines_deleted+0.1),
				data=met10, family=poisson,
				subset=(multi_commenters_in_pr == 0) & (n_comments_in_pr > 0) & (files_in_pr > 1))
# Primary model
met_mod=glm(n_comments ~ n_commenters+I(n_commenters^2)+is_test+
				# cum_log_la+ # log prev_la is a better fit
				log(prev_la+0.1)+
				prev_comments+
				file_position+log(lines_added+0.1)+log(lines_deleted+0.1),
				data=met10, family=poisson,
				subset=(n_comments_in_pr > 0) & (files_in_pr > 1))
summary(met_mod)

exp(coef(met_mod))
#              (Intercept)             n_commenters        I(n_commenters^2) 
#                0.1265445                2.3758607                0.9279686 
#              is_testtrue       log(prev_la + 0.1)            prev_comments 
#                0.5492628                0.9459531                1.0156244 
#            file_position   log(lines_added + 0.1) log(lines_deleted + 0.1) 
#                0.8416546                1.6258617                1.0640887 

met_nls=nls(n_comments ~ a+b*n_commenters^c, data=met10, start=c(a=1, b=1, c=1.2))
summary(met_nls)

mean_fe=ddply(met10, .(file_extension), function(df) mean(df$file_position/df$files_in_pr))

top_mean=subset(mean_fe, file_extension %in% top_fe)

pr_totals=ddply(met10, .(pr_id), function(df)
				{
				return(data.frame(total_comments=sum(df$n_comments),
							total_commenters=sum(df$n_commenters),
							num_files=df$files_in_pr[1],
							total_add_lines=sum(df$lines_added),
							total_del_lines=sum(df$lines_deleted)))
				})

plot(pr_totals$total_commenters, pr_totals$total_del_lines, log="y")
plot(pr_totals$total_add_lines, pr_totals$total_del_lines, log="xy")


# Experiment

expRes=read.csv("./2208.04259/validParticipantsPublic.csv", as.is=TRUE)


expRes$duration_CR=as.double(difftime(expRes$cr1end, expRes$cr1Start, units = "mins"))

# From results_analysis.R
jobs=c("Software developer",
	"Penetration tester",
	"Bug bounty hunter",
	"Software architect",
	"IT operations",
	"Project manager",
	"Researcher/Professor",
	"PhD student",
	"Other")

expRes$job=mapvalues(expRes$question_demo5, 0:8, jobs)


# Create one row per subject answer
mk_subj_info=function(df)
{
# bug_x_validated_final set to 0 if found, and 1 if not found
a1=df
a1$fault="CC"
a1$found_fault=(df$bug_1_validated_final == 0) # found defect CC
a1$is_first=(df$setexperimentCRtype_fixed == "files_experiment1_bf")

b1=df
b1$fault="MB"
b1$found_fault=(df$bug_2_validated_final == 0) # found defect MB
b1$is_first=(df$setexperimentCRtype_fixed == "files_experiment1_bl")

return(rbind(a1, b1))
}


# Map data from wide to long form
answers=ddply(expRes, .(participant), mk_subj_info)


# Workhorse approach, to check things out
exp_mod=glm(found_fault ~ is_first
				# + fault
				# + professional
				+ java
				# + job
				# + year.cr
				# + often.programming
				# + often.cr
				# + duration_CR
				# + question_interruption_time
				,
		family=binomial(logit), data=answers)
summary(exp_mod)

# The correct way of handling multiple answers from same subject
exp_lme=glmer(found_fault ~ is_first+(1 | participant)
				# + fault
				# + professional
				+ java
				# + job
				# + year.cr
				# + often.programming
				# + often.cr
				# + duration_CR
				# + question_interruption_time
				,
		family=binomial(logit), data=answers)
summary(exp_lme)
 

